package com.zswl.dataservicestarter.service;

import com.zswl.dataservicestarter.dao.DataServiceGoodsDao;
import com.zswl.dataservicestarter.dao.DataServiceOrderInfoDao;
import com.zswl.dataservicestarter.dao.DataServicePaymentInfoDao;
import com.zswl.dataservicestarter.domain.DataServiceGoods;
import com.zswl.dataservicestarter.domain.DataServiceOrderInfo;
import com.zswl.dataservicestarter.domain.DataServicePaymentInfo;
import com.zswl.dataservicestarter.model.DataServiceGoodsModel;
import com.zswl.dataservicestarter.model.DataServiceOrderInfoModel;
import com.zswl.dataservicestarter.model.DataServicePaymentInfoModel;
import com.zswl.dataservicestarter.type.DataServiceOrderType;
import com.zswl.dataservicestarter.utils.CommonUtil;
import com.zswl.dataservicestarter.utils.result.ResultContent;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * 订单信息 管理
 *
 * @author TRX
 * @date 2024/3/25
 */
@Service
public class DataServiceOrderInfoService {

    @Autowired
    DataServiceOrderInfoDao orderInfoDao;

    @Autowired
    DataServiceGoodsDao dataServiceGoodsDao;

    @Autowired
    DataServicePaymentInfoDao paymentInfoDao;

    /**
     * 从平台同步订单信息到 数据服务
     *
     * @param model
     * @return
     */
    @Transactional
    public ResultContent saveOrderInfoFromPlatform(DataServiceOrderInfoModel model) {
        // 验证数据完整性
        if (StringUtils.isEmpty(model.getEpId())) {
            return ResultContent.buildFail("商铺ID不能为空");
        }
        if (StringUtils.isEmpty(model.getOrderNo())) {
            return ResultContent.buildFail("订单号不能为空");
        }
        if (model.getShopInfo() == null) {
            return ResultContent.buildFail("店铺信息为空");
        }
        if (CommonUtil.bigDecimalIsEmpty(model.getTotalAmount())) {
            return ResultContent.buildFail("商品总价格不能为空");
        }
        String orderNo = model.getOrderNo();
        DataServiceOrderInfo tempOrderInfo = orderInfoDao.findTopByOrderNo(orderNo);
        if (ObjectUtils.isNotEmpty(tempOrderInfo)) {
//            return ResultContent.buildFail(String.format("%s 订单号已存在", model.getOrderNo()));
        }
        if (CommonUtil.bigDecimalIsEmpty(model.getDiscountAmount())) {
            return ResultContent.buildFail("优惠价格不能为空");
        }
        if (CommonUtil.bigDecimalIsEmpty(model.getPayAmount())) {
            return ResultContent.buildFail("实际支付价格不能为空");
        }

        // 订单管理的商品信息
        List<DataServiceGoodsModel> goodsList = model.getGoodsList();
        if (ObjectUtils.isEmpty(goodsList)) {
            return ResultContent.buildFail("商品不能为空");
        }
        List<DataServiceGoods> goodsList1 = new ArrayList<>();
        for (DataServiceGoodsModel goodsModel : goodsList) {
            DataServiceGoods goods = new DataServiceGoods();
            if (StringUtils.isEmpty(goodsModel.getGoodsId())) {
                return ResultContent.buildFail(String.format("商品id不能为空"));
            }
            if (goodsModel.getGoodsInfo() == null) {
                return ResultContent.buildFail(String.format("商品信息不能为空"));
            }

            BeanUtils.copyProperties(goodsModel, goods);
            goods.setOrderNo(orderNo);

            goodsList1.add(goods);
        }

        // 支付信息
        DataServicePaymentInfoModel paymentInfoModel = model.getPayment();
        if (StringUtils.isEmpty(paymentInfoModel.getPaymentTime())) {
            return ResultContent.buildFail("支付时间为空");
        }
        DataServicePaymentInfo paymentInfo = new DataServicePaymentInfo();
        BeanUtils.copyProperties(paymentInfoModel, paymentInfo);
        paymentInfo.setOrderNo(orderNo);

        dataServiceGoodsDao.saveAll(goodsList1);
        paymentInfoDao.save(paymentInfo);

        // 保存响应数据
        DataServiceOrderInfo orderInfo = new DataServiceOrderInfo();
        BeanUtils.copyProperties(model, orderInfo);
        orderInfo.setGoodsList(goodsList1);
        orderInfo.setPayment(paymentInfo);
        orderInfo.setOrderType(DataServiceOrderType.Platform);
        orderInfo.setIsSync2Platform(Boolean.FALSE);
        orderInfo.setIsSync2Shop(Boolean.FALSE);
        int hasCode = CommonUtil.getEntityHashCode(orderInfo);
        orderInfo.setHashCode(hasCode);

        orderInfoDao.save(orderInfo);

        return ResultContent.buildSuccess(hasCode);
    }

    /**
     * 根据订单号查询订单信息
     *
     * @param orderNo
     * @return
     */
    public ResultContent<DataServiceOrderInfo> getOrderInfoByOrderNo(String orderNo) {
        if (StringUtils.isEmpty(orderNo)) {
            return ResultContent.buildFail("订单号为空");
        }
        DataServiceOrderInfo orderInfo = orderInfoDao.findTopByOrderNo(orderNo);

        return ResultContent.buildSuccess(orderInfo);
    }

    /**
     * 更加HashCode验证支付信息是否存在
     *
     * @param hashCode
     * @return
     */
    public ResultContent validateDataByHashCode(int hashCode) {
        DataServiceOrderInfo orderInfo = orderInfoDao.findTopByHashCode(hashCode);
        if (ObjectUtils.isEmpty(orderInfo)) {
            return ResultContent.buildFail("对应数据未找到");
        }
        return ResultContent.buildSuccess();
    }

    /**
     * 编辑信息是否同步到平台
     *
     * @param dataId
     * @return
     */
    public ResultContent markSyncPlatform(String dataId) {
        int a = 100;
        return ResultContent.buildSuccess();
    }

}
