package com.zswl.dataservicestarter.utils;

import com.google.common.collect.Lists;
import com.zswl.dataservicestarter.domain.base.DataServiceSuperEntity;
import com.zswl.dataservicestarter.utils.exception.BusinessException;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.bson.types.Decimal128;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;

import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.*;

/**
 * 工具类方法
 *
 * @author TRX
 * @date 2024/3
 */
@Slf4j
public class CommonUtil {

    private static final List<String> outKeys = List.of("id", "createTime", "updateTime", "remark",
            "hashCode", "isSync2Platform", "syncTime", "isSync2Shop");

    public static <T extends DataServiceSuperEntity> int getEntityHashCode(T entity) {
        int code = 0;
        if (ObjectUtils.isEmpty(entity)) {
            return code;
        }
        Field[] fields = entity.getClass().getDeclaredFields();
        for (Field field : fields) {
            String name = field.getName();
            Class type = field.getType();
//            log.info("字段: {} -> {}", name, type);
            if (outKeys.contains(name)) {
                continue;
            }
            int _code = 0;
            field.setAccessible(true);
            try {
                Object obj = field.get(entity);
                if (obj != null) {
                    boolean isSuper = false;
                    if (obj instanceof DataServiceSuperEntity) {
                        isSuper = true;
                    }
                    if (!isSuper && obj instanceof List) {
                        List<Object> objects = (List<Object>) obj;
                        isSuper = objects.stream().anyMatch(element -> element instanceof DataServiceSuperEntity);
                    }
                    if (!isSuper) {
                        _code = obj.hashCode();
                    }
                }
//                log.info("值: {} {} ", obj, _code);
            } catch (Exception e) {
                e.printStackTrace();
                throw new BusinessException(String.format("HashCode出错: %s", e.getMessage()));
            }
            code += _code;
        }
        return code;
    }

    /**
     * 生成指定长度的随机码（由数字构成）
     *
     * @param length
     * @return
     */
    public static String randomCode(int length) {
        StringBuilder str = new StringBuilder();
        Random random = new Random();
        for (int i = 0; i < length; i++) {
            str.append(random.nextInt(10));
        }
        return str.toString();
    }

    /**
     * 生产UUID
     *
     * @return
     */
    public static String UUID() {
        return UUID.randomUUID().toString().replaceAll("-", "");
    }

    /**
     * 创建空分页对象
     *
     * @param pageable
     * @param cls
     * @param <T>
     * @return
     */
    public static <T> Page<T> buildEmptyPage(Pageable pageable, Class<T> cls) {
        return new PageImpl<T>(Lists.newArrayList(), pageable, 0);
    }

    public static Long getLongZero(Long number) {
        if (number == null) {
            return Long.valueOf(0);
        }
        return number;
    }

    /**
     * 判断long值是否为空：为 null 或 0 时都为空
     *
     * @param longValue
     * @return
     */
    public static boolean longIsEmpty(Long longValue) {
        if (longValue == null || longValue <= 0) {
            return true;
        }
        return false;
    }

    /**
     * 判断bigDecimal值是否为空：为 null 或 0 时都为空
     *
     * @param bigDecimal
     * @return
     */
    public static boolean bigDecimalIsEmpty(BigDecimal bigDecimal) {
        if (bigDecimal == null || bigDecimal.compareTo(BigDecimal.ZERO) <= 0) {
            return true;
        }
        return false;
    }

    /**
     * 2个Long是否相等
     *
     * @param val1
     * @param val2
     * @return
     */
    public static boolean longIsEqual(Long val1, Long val2) {
        if (val1 == null || val2 == null) {
            return false;
        }
        if (val1.longValue() == val2.longValue()) {
            return true;
        }
        return false;
    }

    /**
     * 判断2个时间 是否是同一天
     *
     * @param time1
     * @param time2
     * @return
     */
    public static boolean isSameDay(Long time1, Long time2) {
        if (time1 == null || time2 == null) {
            return false;
        }
        if (DateUtils.paresTime(time1, DateUtils.pattern).equals(DateUtils.paresTime(time2, DateUtils.pattern))) {
            return true;
        }
        return false;
    }

    public static BigDecimal getBigDecimalByLong(Long number) {
        if (number != null) {
            return BigDecimal.valueOf(number);
        }
        return BigDecimal.ZERO;
    }

    public static String getCollectionName(Class domainClass) {
        if (domainClass != null) {
            String className = domainClass.getSimpleName();
            String collectionName = className.replace(className.charAt(0), String.valueOf(className.charAt(0)).toLowerCase().toCharArray()[0]);
            return collectionName;
        } else {
            return null;
        }
    }

    public static String getCollectionName(String collectionName) {
        if (StringUtils.isNotEmpty(collectionName)) {
            String[] arr = collectionName.split(".");
            if (arr.length > 1) {
                collectionName = arr[(arr.length - 1)];
            }
            collectionName = collectionName.replace(collectionName.charAt(0), String.valueOf(collectionName.charAt(0)).toLowerCase().toCharArray()[0]);
            return collectionName;
        } else {
            return null;
        }
    }

    /**
     * obj对象转 BigDecimal
     *
     * @param obj
     * @return
     */
    public static BigDecimal getBigDecimalByObj(Object obj) {
        BigDecimal count = BigDecimal.ZERO;
        if (obj != null) {
            if (obj instanceof Integer) {
                count = new BigDecimal((Integer) obj);
            } else if (obj instanceof Decimal128) {
                Decimal128 countDecimal = (Decimal128) obj;
                count = countDecimal.bigDecimalValue();
            } else if (obj instanceof BigDecimal) {
                count = (BigDecimal) obj;
            } else if (obj instanceof Long) {
                count = new BigDecimal((Long) obj);
            }
        }
        return count;
    }

    /**
     * BigDecimal 保留四位小数
     *
     * @param bigDecimal
     * @return
     */
    public static BigDecimal getBigDecimalScale4(BigDecimal bigDecimal) {
        if (bigDecimal != null) {
            String ss = bigDecimal.setScale(4, RoundingMode.HALF_UP).stripTrailingZeros().toPlainString();
            bigDecimal = new BigDecimal(ss);
            return bigDecimal;
        }
        return BigDecimal.ZERO;
    }

    /**
     * 向上取整,如：16.251、16.256 -> 16.25
     *
     * @param bigDecimal
     * @return
     */
    public static BigDecimal getBigDecimalScale2(BigDecimal bigDecimal) {
        if (bigDecimal != null) {
            String ss = bigDecimal.setScale(2, RoundingMode.HALF_UP).stripTrailingZeros().toPlainString();
            bigDecimal = new BigDecimal(ss);
            return bigDecimal;
        }
        return BigDecimal.ZERO;
    }

    public static BigDecimal getBigDecimalScale2Floor(BigDecimal bigDecimal) {
        if (bigDecimal != null) {
            String ss = bigDecimal.setScale(2, RoundingMode.FLOOR).stripTrailingZeros().toPlainString();
            bigDecimal = new BigDecimal(ss);
            return bigDecimal;
        }
        return BigDecimal.ZERO;
    }

    /**
     * 不保留小数，， 四舍五入
     *
     * @param bigDecimal
     * @return
     */
    public static BigDecimal getBigDecimalScale0(BigDecimal bigDecimal) {
        if (bigDecimal != null) {
            String ss = bigDecimal.setScale(0, RoundingMode.HALF_UP).stripTrailingZeros().toPlainString();
            bigDecimal = new BigDecimal(ss);
            return bigDecimal;
        }
        return BigDecimal.ZERO;
    }

    /**
     * 不要小数，，而且 向下取数 如：16.2455、16.874 -> 16
     *
     * @param bigDecimal
     * @return
     */
    public static BigDecimal getBigDecimalScale0Floor(BigDecimal bigDecimal) {
        if (bigDecimal != null) {
            String ss = bigDecimal.setScale(0, RoundingMode.FLOOR).stripTrailingZeros().toPlainString();
            bigDecimal = new BigDecimal(ss);
            return bigDecimal;
        }
        return BigDecimal.ZERO;
    }


    public static Map<String, Object> filterData(Collection<Map<String, Object>> datas, List<String> props) {
        Map<String, Object> useDataMap = new HashMap<>(props.size());
        datas.forEach(actDataMap -> {
            for (String prop : props) {
                if (actDataMap.containsKey(prop)) {
                    useDataMap.put(prop, actDataMap.get(prop));
                }
            }
        });
        return useDataMap;
    }

    /**
     * BigDecimal 是否含有小数，如：12.202 true  12.0000 false
     *
     * @param number
     * @return
     */
    public static boolean bigDecimalHasDec(BigDecimal number) {
        boolean b = false;
        if (number != null) {
            BigDecimal number1 = BigDecimal.valueOf(number.intValue());
            if (number1.compareTo(number) != 0) {
                b = true;
            }
        }
        return b;
    }

    /**
     * 判断 BigDecimal 是否是最小值
     *
     * @param number
     * @return
     */
    public static boolean bigDecimalIsSmall(BigDecimal number) {
        if (number.compareTo(BigDecimal.valueOf(0.02)) <= 0) {
            return true;
        }
        return false;
    }

    public static boolean treeIdIsTop(String id) {
        if (StringUtils.isEmpty(id) || id.equals(ITree.ROOT_ID)) {
            return true;
        }
        return false;
    }

    /**
     * 小时转为分钟，，保留整数 （最小 1 分钟）
     *
     * @param hours 1.5
     * @return 90
     */
    public static int trun2Mins(BigDecimal hours) {
        int mins = 0;
        if (hours != null) {
            BigDecimal number = getBigDecimalScale0(hours.multiply(BigDecimal.valueOf(60)));
            mins = number.intValue();
            if (mins <= 0) {
                mins = 1;
            }
        }
        return mins;
    }


}
